//  *****************************************************************************************************************
//  *                                                                                                               *
//  *                                         SpikenzieLabs.com                                                     *
//  *                                                                                                               *
//  *                                 SPLixel Controller Library Examples                                           *
//  *                                                                                                               *
//  *****************************************************************************************************************
//
// BY: MARK DEMERS - SpikenzieLabs
// March 2014
// VERSION 1.0
//
// Brief:
// Examples of the SpikenzieLabs SPLixel Controller Library.
// 
// The SpikezieLabs SPLixel Controller is a serial to WS2811, WS2812 and WS2812B based LED and LED strip hardware 
// interface. The SPLixel Controller does all of the heavy lifting and timing needed to control the WS281x series
// LEDs. The beauty here is that after your microcontroller sends a serial command to the SPLixel Controller your
// microcontroller is compleatly free to perform other taskes and does not have to deal with the LEDs.
//
// This library is not needed to use the SPLixel Controllers, but is provided to make using it on an Arduino even
// more simple. If you are interested in using the SPLixel Controllers with other non-arduino microcontroller or
// directly with a computer, please see the related documents on our web sight for a list of serial commands that
// will work with the SPLixel Controllers.
//
// LEGAL:
// This code is provided as is. No guaranties or warranties are given in any form. It is your responsibilty to 
// determine this codes suitability for your application.
// 

#include <SPLixel.h>                                // Include the SPLixel Library

SPLixel SPLixBasic(64);                             // Set the number of LEDs connected to the SPLixel Controller

int maxleds = 64;                                   // Only needed for this DEMO
byte temp = 0;                                      // Only needed for this DEMO


void setup()
{
  delay(2000);
  SPLixBasic.begin();                              // Open connection to the SPLixel Controller
  delay(500);
  SPLixBasic.clear();                              // Clear any junk colors
  delay(1000);

  randomSeed(analogRead(0));                       // Only needed for this DEMO
}

void loop()
{

  // -------------------------------------------------------------------------------------------------------------------
  // Rainbow
  // -------------------------------------------------------------------------------------------------------------------
  //
  // Rainbox will fill in the LEDs with rainbow like colors.
  // 
  // rainbow(start LED, end LED) 
  // rainbow(start LED, end LED, refresh) 
  //
  // LEDs are numbered from 1 to 64 (or less) 
  // If used, refresh is 0 or 1 where 1 does not refesh LEDs and 0 does.
  //
  // NOTE: Rainbow is based on multiples of 3 (since the LEDs are red,greeb,blue). For best results make your higher (end LED)
  //       a multiple of 3. Eg. If you have an 8 LED strip, use SPLixBasic.rainbow(1,9). Max must not exceed 64.
  //
  SPLixBasic.rainbow(1,maxleds);
  delay(3000);

  // Clear - Turn off all the LEDs
  SPLixBasic.clear();
  delay(1000);

  // -------------------------------------------------------------------------------------------------------------------
  // All
  // -------------------------------------------------------------------------------------------------------------------
  //
  // Color All LEDs the same
  //
  // all(red,green,blue)
  //
  // red, green and blue are vales from 0 to 255
  // Fill in all of the LEDs with same color, upto the maximum number of LEDs set 
  //
  SPLixBasic.all(0,40,20); 
  delay(2000);
  SPLixBasic.all(20,0,20);
  delay(2000);
  SPLixBasic.all(80,0,0);
  delay(2000);

  // -------------------------------------------------------------------------------------------------------------------
  // Draw
  // -------------------------------------------------------------------------------------------------------------------
  //
  // Will light a LED and refresh all the LEDs right away
  //
  // draw(LED number, red, green, blue)
  //
  // Where LED number is from 1 to to 64 (or less) and red, green and blue are vales from 0 to 255
  //
  // NOTE: If SPLixel Controller was instantiated with a LED number less then that passed in draw, the higher value
  //       will become the new size of the LED "strip"
  //
  // NOTE: If your strip is 64 LEDs long the SPLixel controller will need 2.06ms to refresh all the LEDs.
  //       We added a delay here, to allow the SPLixel Controller to finish upding the LED before a new command is 
  //       started.  For faster performace see "buffer" below in the examples.
  //
  SPLixBasic.draw(1,0,0,200);
  delay(3);
  SPLixBasic.draw(2,0,50,50);
  delay(3);
  SPLixBasic.draw(3,50,0,50);
  delay(3);
  SPLixBasic.draw(4,50,50,0);
  delay(3);
  SPLixBasic.draw(5,25,75,0);
  delay(3);
  SPLixBasic.draw(6,0,50,100);
  delay(3);
  SPLixBasic.draw(7,75,30,0);
  delay(3);
  SPLixBasic.draw(8,10,20,30);

  delay(2000);

  // -------------------------------------------------------------------------------------------------------------------
  // ShiftUp & ShiftDown
  // -------------------------------------------------------------------------------------------------------------------
  // 
  // Will shift all the LEDs either up or down the LED strip, one LED at a time. LEDs that fall off the top
  // or bottom are redrawn on the other end of the LED strip. Will run continuously.
  // 
  // shiftup(delay speed);
  // shiftdown(delay speed);
  //
  // Where delay speed is a value from 1 to 255, so that the higher the value the slower the shifting of the LED color
  //
  SPLixBasic.shiftup(255);
  delay(3000);
  SPLixBasic.shiftdown(20);
  delay(3000);  
  SPLixBasic.shiftup(3);
  delay(3000);
  SPLixBasic.shiftdown(200);
  delay(3000);  

  // -------------------------------------------------------------------------------------------------------------------
  // ShiftEight
  // -------------------------------------------------------------------------------------------------------------------
  //
  // ShiftEight shift all of the LED color by 8 LEDs, only one time. LEDs that fall off the top are redrawn on the other
  // end of the LED strip. Optionally, the shifted LEDs color can either be refreshed or not. (may make for smoother 
  // animations.) Shifteight may be helpful when working with 64 LEDs configured into an 8x8 LED matrix and moving columns
  // is desired.
  //
  // shifteight(refresh)
  //
  // Where refresh is 0 to not refesh or 1 to refresh right away.
  //
    for(int i =1; i <4; i++)
  {
    SPLixBasic.shifteight(1);
    delay(1000);
  }
  
     for(int i =1; i <4; i++)
  {
    SPLixBasic.shifteight(0);
    delay(100);
  }

  SPLixBasic.refresh();
  delay(3000);

  SPLixBasic.clear();
  delay(1000);    

  // -------------------------------------------------------------------------------------------------------------------
  // Buffer
  // -------------------------------------------------------------------------------------------------------------------
  //
  // Buffer is similar to draw in that it is used to update the color of a single LED at a time. It differs in that buffer
  // only updates the color value stored in the ram memory of the SPLixel Controller, and does not send the data out to the
  // LEDs. Depending on you application, buffer can be much faster than draw.
  //
  // buffer(LED number, red, green, blue)
  //
  // Where LED number is from 1 to to 64 (or less) and red, green and blue are vales from 0 to 255
  //
  // When using buffer (vs draw) to force the SPLixel Controller to update the colors displayed on the LEDs, issue the refresh command.
  //
  // refresh()
  //
  // NOTE: Other commands will also force the display of the new color values, such as shiftup, shiftdown, dimmer etc ...
  //
  // Draw to memmory only - Buffer
  for(byte i =1; i <(maxleds-2); i=i+3)
  {
    SPLixBasic.buffer(i,i,0,0);
    delay(3);
    SPLixBasic.buffer(i+1,0,i,0);
    delay(3);
    SPLixBasic.buffer(i+2,0,0,i);
    delay(3);
  }  
  SPLixBasic.refresh();
  delay(3000);
  
  // -------------------------------------------------------------------------------------------------------------------
  // Shift-up COUNT and Shift-Down COUNT
  // -------------------------------------------------------------------------------------------------------------------
  //
  // shiftupcount and shiftdowncount are similar to shiftup and shiftdown, but there is an extra argument that sets the
  // number of LEDs to shift.
  //
  // shiftup(delay speed, count);
  // shiftdown(delay speed, count);
  //
  // Where delay speed is a value from 0 to 255, so that the higher the value the slower the shifting of the LED color
  // and count is from 1 to 255 for the finite number of LEDs to shift.
  //
  SPLixBasic.shiftup(200, 20);
  delay(1000);
  SPLixBasic.shiftdown(20, 20);
  delay(3000);
  
  // Clear - Turn off all the LEDs
  SPLixBasic.clear();
  delay(1000);
  
  // -------------------------------------------------------------------------------------------------------------------
  // Shiftin
  // -------------------------------------------------------------------------------------------------------------------
  //
  // shiftin will set the first LED to the color specified and all the other LEDs will shift away. The color of the last
  // LED will be lost.
  //
  // shiftin(red,green,blue)
  // 
  // Where red, green and blue are vales from 0 to 255
  //
  
  for(int i =1; i <maxleds; i++)
  {
    SPLixBasic.shiftin(0,0,i);
    delay(100);
  }
  delay(3000);

  // -------------------------------------------------------------------------------------------------------------------
  // Larson
  // -------------------------------------------------------------------------------------------------------------------
  //
  // Creates a Larson Scanner type effect. The "eye" will bounce from the first LED to the highest LED.
  // 
  // larson(red,green,blue,speed)
  //
  // Where red, green and blue are vales from 0 to 255 for the color of the scanning "eye" and speed (1 to 255) is the 
  // speed of the delay used in the scanning effect.
  //
  SPLixBasic.all(0,5,2);                                // Fill in a background color (or not)
  SPLixBasic.larson(255,0,0,30);
  delay(9000);

  // Clear - Turn off all the LEDs
  SPLixBasic.clear();
  delay(1000);
  
  // -------------------------------------------------------------------------------------------------------------------
  // Blink
  // -------------------------------------------------------------------------------------------------------------------
  //
  // Will blink all of the LEDs by turning them on and off.
  // 
  // blink(speed)
  //
  // Where speed (1 to 255) is the speed of the delay. Higher numbers are slower blinking.
  //
  // NOTE: The delay used in blink is not the same as used elsewhere and is much slower, so the speed values much lower.
  //
  SPLixBasic.all(255,221,0);
  SPLixBasic.blink(255); 
  delay(5000);

  // Clear - Turn off all the LEDs
  SPLixBasic.clear();
  delay(1000);    

  // -------------------------------------------------------------------------------------------------------------------
  // Bar
  // -------------------------------------------------------------------------------------------------------------------
  //
  // Bar makes a LED bar graph type effect. Usefull for a "dance party" where music volume could be sampled to pump
  // the LED bar graph.
  // Starting at LED 1 the SPLixel Controller fills in the bar color until it gets to the top LED. The top LED is draw,
  // then the remaining LEDs are filled with whatever color is in memory (this is why in the example below we color all
  // LEDs to make a background color.
  // The speed for the rise and fall of the LED bar graph is settable.
  //
  // barcolor(red,green,blue)
  // bartopcolor(red,green,blue)
  // baraction(height, up speed, down speed)
  //
  // Where; red, green and blue are vales from 0 to 255 for the color
  //        height is the cureent "top" of the LED bar graph (a value from 1 to 64 (or less if you have less LEDs)
  //        up speed and down speed (from 1 to 255) is the delay used in increasing decreasing the bar graph, where
  //        higher numbers are slower.
  //
  SPLixBasic.all(0,0,40);
  delay(3);
  SPLixBasic.barcolor(0,80,0);
  SPLixBasic.bartopcolor(255,0,0);
  delay(800);
  SPLixBasic.baraction(maxleds/4,10,20);
  delay(400);

  for(int i =1; i <50; i++)
  {
    temp = random(maxleds/4, (maxleds/6)*5);
    SPLixBasic.baraction(temp,10,20);
    delay(400);
  }

  // Clear - Turn off all the LEDs
  SPLixBasic.clear();
  delay(1000);  

  // -------------------------------------------------------------------------------------------------------------------
  // Dimmer
  // -------------------------------------------------------------------------------------------------------------------
  // 
  // Dimmer will lower the brightness of the LEDs while mantaining the same color mix buy scalling down the red,green and blue
  // values based on a percentage.
  //
  // dimmer(percent)
  //
  // Where percent is a value from 1 to 100
  //
  // NOTE: In this example the non-LED refreshing version of rainbow is used so that there is not a flash of bight color before
  //       the dimmer function acts on the LEDs
  //
  // Rainbow and Dimmer
  
  SPLixBasic.rainbow(1,maxleds,1);          // Do not update LEDs yet
  SPLixBasic.dimmer(20);                    // after they are dimmed the LEDs will refresh

  SPLixBasic.shiftup(6);
  delay(3000);

  // -------------------------------------------------------------------------------------------------------------------
  while(1);                                 // Stop here

}





